from abc import ABC, abstractmethod
from typing import Any, final

from baml_agents._baml_client_proxy._hooks._base_hook import (
    BaseBamlHookAsync,
    BaseBamlHookContext,
    BaseBamlHookSync,
)


@final
class OnPartialResponseParsedHookContext(BaseBamlHookContext):
    partial_result: Any


class OnPartialResponseParsedHookAsync(BaseBamlHookAsync, ABC):
    @abstractmethod
    async def on_partial_response_parsed(
        self,
        *,
        ctx: OnPartialResponseParsedHookContext,
    ) -> None:
        pass


class OnPartialResponseParsedHookSync(BaseBamlHookSync, ABC):
    @abstractmethod
    def on_partial_response_parsed(
        self,
        *,
        ctx: OnPartialResponseParsedHookContext,
    ) -> None:
        pass



from collections.abc import Mapping
from typing import Any

from baml_agents._agent_tools._json_schema_to_baml_source._model_to_baml_source import (
    BamlModelToBamlSourceConverter,
)

from ._json_to_model import (
    JsonSchemaToBamlModelConverter,
    JsonSchemaToBamlModelConverterConfig,
)


def json_schema_to_baml_source(
    class_name,
    json_schema: str | Mapping[str, Any],
    *,
    schema_to_model_config: JsonSchemaToBamlModelConverterConfig | None = None,
):
    schema_to_model = JsonSchemaToBamlModelConverter(
        json_schema, class_name, config=schema_to_model_config
    )
    baml_models = schema_to_model.convert()
    model_to_source = BamlModelToBamlSourceConverter(baml_models)
    baml_source = model_to_source.generate()
    return baml_source




def snake_to_pascal(name: str) -> str:
    name = pascal_to_snake(name)
    """Convert snake_case or kebab-case to PascalCase."""
    return "".join(word.capitalize() for word in name.replace("-", "_").split("_"))


def pascal_to_snake(name: str) -> str:
    """
    Convert PascalCase or camelCase to snake_case.
    """
    name = name.replace(" ", "_")
    if not name:
        return ""
    chars = []
    for i, c in enumerate(name):
        if (
            c.isupper()
            and i != 0
            and (
                not name[i - 1].isupper()
                or (i + 1 < len(name) and not name[i + 1].isupper())
            )
        ):
            chars.append("_")
        chars.append(c.lower())
    return "".join(chars)




from typing import Any


def merge_dicts_no_overlap(
    a: dict[Any, Any], b: dict[Any, Any], error_message: str | None = None
) -> dict[Any, Any]:
    """Merge two dicts, error on key collisions."""
    overlap = set(a) & set(b)
    if overlap:
        error_message = "Collision on keys"
        raise KeyError(f"{error_message}: {overlap}")
    return {**a, **b}



from baml_agents._agent_tools._action import Action
from baml_agents._agent_tools._mcp import ActionRunner
from baml_agents._agent_tools._str_result import Result
from baml_agents._agent_tools._tool_definition import McpToolDefinition
from baml_agents._agent_tools._utils._baml_utils import display_prompt, get_prompt
from baml_agents._baml_client_proxy._baml_client_proxy import BamlClientProxy
from baml_agents._baml_client_proxy._hook_engine import HookEngineAsync, HookEngineSync
from baml_agents._baml_client_proxy._hooks._base_hook import (
    BaseBamlHook,
    BaseBamlHookContext,
)
from baml_agents._baml_client_proxy._hooks._implementations._test_generator import (
    BamlTestGeneratorHook,
)
from baml_agents._baml_client_proxy._hooks._implementations._with_options import (
    WithOptions,
)
from baml_agents._baml_client_proxy._hooks._on_after_call_success_hook import (
    OnAfterCallSuccessHookAsync,
    OnAfterCallSuccessHookContext,
    OnAfterCallSuccessHookSync,
)
from baml_agents._baml_client_proxy._hooks._on_before_call_hook import (
    OnBeforeCallHookAsync,
    OnBeforeCallHookContext,
    OnBeforeCallHookSync,
)
from baml_agents._baml_client_proxy._hooks._on_error_hook import (
    OnErrorHookAsync,
    OnErrorHookContext,
    OnErrorHookSync,
)
from baml_agents._baml_client_proxy._hooks._on_partial_response_parsed_hook import (
    OnPartialResponseParsedHookAsync,
    OnPartialResponseParsedHookContext,
    OnPartialResponseParsedHookSync,
)
from baml_agents._baml_client_proxy._hooks._types import Mutable
from baml_agents._baml_client_proxy._with_hooks import with_hooks
from baml_agents._baml_clients._with_baml_client import with_baml_client
from baml_agents._baml_clients._with_model import BamlModelConfig, with_model
from baml_agents._project_utils._get_root_path import get_root_path
from baml_agents._project_utils._init_logging import init_logging
from baml_agents._utils._must import must
from baml_agents._utils._sole import sole

__version__ = "0.38.2"
__all__ = [
    "Action",
    "ActionRunner",
    "BamlClientProxy",
    "BamlModelConfig",
    "BamlTestGeneratorHook",
    "BaseBamlHook",
    "BaseBamlHookContext",
    "HookEngineAsync",
    "HookEngineSync",
    "McpToolDefinition",
    "Mutable",
    "OnAfterCallSuccessHookAsync",
    "OnAfterCallSuccessHookContext",
    "OnAfterCallSuccessHookSync",
    "OnBeforeCallHookAsync",
    "OnBeforeCallHookContext",
    "OnBeforeCallHookSync",
    "OnErrorHookAsync",
    "OnErrorHookContext",
    "OnErrorHookSync",
    "OnPartialResponseParsedHookAsync",
    "OnPartialResponseParsedHookContext",
    "OnPartialResponseParsedHookSync",
    "Result",
    "WithOptions",
    "display_prompt",
    "get_prompt",
    "get_root_path",
    "init_logging",
    "must",
    "sole",
    "with_baml_client",
    "with_hooks",
    "with_model",
]



from typing import Any

from pydantic import BaseModel


# The core recursive formatting function
def _format_baml_value(value: Any, indent_level: int) -> str:
    """
    Recursively formats a Python value into BAML syntax string.
    The returned string includes necessary indentation starting at indent_level.
    Handles primitives, lists, dicts, and Pydantic models.
    """
    indent_space = "  " * indent_level
    content_indent_space = "  " * (indent_level + 1)

    if isinstance(value, BaseModel):
        # Format model as dict at the same indent level
        return _format_baml_value(value.model_dump(mode="json"), indent_level)

    if isinstance(value, dict):
        if not value:
            return "{}"  # Empty dict inline
        items = []
        for k, v in value.items():
            formatted_key = str(k)
            # Format value recursively, starting at the content level
            formatted_value = _format_baml_value(v, indent_level + 1)
            # Combine key and value. Primitive values returned by _format_baml_value
            # have no leading space. Complex values ({...} or [...]) might,
            # but combining them directly after the key space is usually correct.
            items.append(f"{content_indent_space}{formatted_key} {formatted_value}")

        # Construct dict string with braces at the current indent level
        return f"{indent_space}{{\n" + "\n".join(items) + f"\n{indent_space}}}"

    if isinstance(value, list):
        if not value:
            return "[]"  # Empty list inline
        items = []
        is_complex_list = any(
            isinstance(item, (dict, BaseModel, list)) for item in value
        )

        for i, item in enumerate(value):
            # Format item recursively, starting at the content level (+1)
            # formatted_item will be a string representing the item,
            # correctly indented to start at level indent_level + 1.
            formatted_item = _format_baml_value(item, indent_level + 1)
            comma = "," if is_complex_list and i < len(value) - 1 else ""
            # Append the already-indented item string and the comma
            items.append(f"{formatted_item}{comma}")

        # Construct list string with brackets at the current indent level
        # Items are joined by newlines; their existing indentation handles alignment.
        return "[\n" + "\n".join(items) + f"\n{indent_space}]"

    # Primitives: Return representation without adding indentation here.
    if isinstance(value, str):
        return f'#"{value}"#' if value else '""'
    if isinstance(value, bool):
        return "true" if value else "false"
    if isinstance(value, (int, float)):
        return str(value)
    if value is None:
        return "null"
    # Fallback for unsupported types
    repr_str = repr(value)
    return f'#"{repr_str}"#' if repr_str else '""'


def get_args_block_str(params):
    args_to_format = {k: v for k, v in params.items() if k != "baml_options"}
    formatted_args = []
    # Arguments inside 'args {}' block start at indent level 1
    args_block_indent_level = 2
    args_block_indent_space = "  " * args_block_indent_level

    for k, v in args_to_format.items():
        # Format the value. The value itself should be formatted
        # as if starting at the same level as the key for BAML style.
        # Let _format_baml_value handle the internal indentation relative to this level.
        formatted_value = _format_baml_value(v, args_block_indent_level)

        # Combine the argument key (at level 1 indent) with the formatted value.
        formatted_args.append(f"{args_block_indent_space}{k} {formatted_value}")

    # Join the formatted argument lines for the args block content
    return "\n".join(formatted_args)



from abc import ABC, abstractmethod

from baml_py.baml_py import FieldType
from baml_py.type_builder import TypeBuilder

from baml_agents._agent_tools._mcp_tool_to_json_schema import mcp_tool_to_json_schema
from baml_agents._agent_tools._tool_definition import McpToolDefinition

from ._abstract_json_schema_to_baml_converter import AbstractJsonSchemaToBamlConverter


class AbstractToolToBamlType(ABC):

    @abstractmethod
    def convert(
        self,
        *,
        tool: McpToolDefinition,
        tb: TypeBuilder,
        baml_tool_id_field: str,
        **_,
    ) -> FieldType:
        pass


class ToolToBamlType(AbstractToolToBamlType):
    def __init__(
        self,
        *,
        schema_converter: AbstractJsonSchemaToBamlConverter,
    ):
        self._converter = schema_converter

    def convert(
        self,
        *,
        tool: McpToolDefinition,
        tb: TypeBuilder,
        baml_tool_id_field: str,
    ) -> FieldType:
        schema = mcp_tool_to_json_schema(tool, tb, baml_tool_id_field)
        return self._converter.convert(schema, tb)




from dataclasses import dataclass, field


@dataclass(frozen=True)
class BamlToolPromptConfig:
    id_field: str = field(
        default="action_id",
        metadata={"description": "Field name for tool ID"},
    )
    tools_field: str = field(
        default="chosen_action",
        metadata={"description": "Field name for tools collection"},
    )




import functools
import inspect
from collections.abc import Callable
from typing import Any


class PassthroughWrapper:
    """
    A wrapper that intercepts attribute access for a given object.
    It distinguishes between regular and async methods and returns
    a corresponding wrapper that simply calls the original method.
    Non-callable attributes are returned directly.
    """

    def __init__(
        self,
        obj: Any,
        /,
        *,
        mutate_args_kwargs: Callable | None = None,
        parent_target=None,
    ):
        # Use a different internal name to avoid potential clashes if wrapping
        # an object that also uses '_ai' or '_obj'.
        object.__setattr__(self, "_passthrough_target", obj)
        self._mutate_args_kwargs = (
            mutate_args_kwargs
            if mutate_args_kwargs is not None
            else lambda args, kwargs, _: (args, kwargs)
        )
        self._parent_target = parent_target

    def __getattribute__(self, name: str) -> Any:
        # 1. Access internal attributes of the wrapper directly.
        # Use object.__getattribute__ to prevent recursion.
        if name == "request":
            parent_target = getattr(self, "_passthrough_target", None)
            return PassthroughWrapper(
                object.__getattribute__(self, "_passthrough_target").request,
                mutate_args_kwargs=self._mutate_args_kwargs,
                parent_target=parent_target,
            )

        if name in {
            "_passthrough_target",
            "_mutate_args_kwargs",
            "_parent_target",
            "_get_baml_function_return_type_name",
            "__class__",
            "__init__",
            "__getattribute__",
            "__setattr__",
            "__delattr__",
            "__dict__",
            "__dir__",
            "__repr__",
            "__str__",
            # Async/Awaitable checks often look for these
            "__await__",
            "__aiter__",
            "__anext__",
            "__aenter__",
            "__aexit__",
        }:
            try:
                # Get attributes of the wrapper itself first
                return object.__getattribute__(self, name)
            except AttributeError:
                # If the wrapper doesn't have it (e.g., __await__),
                # fall through to get it from the target below.
                pass

        # 2. Get the target object and the requested attribute from it.
        target = object.__getattribute__(self, "_passthrough_target")
        try:
            attr = getattr(target, name)
        except AttributeError:
            # If the target doesn't have the attribute, raise AttributeError naturally.
            # Re-raise the specific error from getattr.
            raise AttributeError(
                f"'{type(target).__name__}' object has no attribute '{name}'"
            ) from None

        # 3. If the attribute is not callable, return it directly.
        if not callable(attr):
            return attr

        mutate_func = object.__getattribute__(self, "_mutate_args_kwargs")

        # 4. If the attribute is callable, determine if it's async or sync.
        if inspect.iscoroutinefunction(attr):

            # Create an ASYNC wrapper
            async def async_wrapper(*args, **kwargs):
                # --- Future extension point (before call) ---
                baml_function_return_type_name = (
                    self._get_baml_function_return_type_name(name)
                )
                mutated_args, mutated_kwargs = mutate_func(
                    args, kwargs.copy(), baml_function_return_type_name
                )
                result = await attr(*mutated_args, **mutated_kwargs)
                # --- Future extension point (after call) ---
                return result

            # Copy metadata
            return functools.wraps(attr)(async_wrapper)

        # Create a SYNC wrapper
        def sync_wrapper(*args, **kwargs):
            # --- Future extension point (before call) ---
            baml_function_return_type_name = self._get_baml_function_return_type_name(
                name
            )
            mutated_args, mutated_kwargs = mutate_func(
                args, kwargs.copy(), baml_function_return_type_name
            )
            result = attr(*mutated_args, **mutated_kwargs)
            # --- Future extension point (after call) ---
            return result

        # Copy metadata
        return functools.wraps(attr)(sync_wrapper)

    # Optional: Proxy __dir__ to make introspection work better
    def __dir__(self):
        target = object.__getattribute__(self, "_passthrough_target")
        # Combine wrapper's dir and target's dir
        return sorted(set(object.__dir__(self)) | set(dir(target)))

    # Optional: Custom repr
    def __repr__(self):
        target = object.__getattribute__(self, "_passthrough_target")
        return f"<_PassthroughWrapper wrapping {target!r}>"

    def _get_baml_function_return_type_name(self, name):
        baml_client = getattr(self, "_parent_target", None) or self
        return getattr(baml_client, name).__annotations__["return"].__name__




from baml_agents.devtools._generate_commit_message._generate_commit_message import (
    generate_commit_message,
)
from baml_agents.devtools._inline_baml_tests import inline_baml_tests
from baml_agents.devtools._update_baml_generator_versions import (
    update_baml_generator_versions,
)

__all__ = [
    "generate_commit_message",
    "inline_baml_tests",
    "update_baml_generator_versions",
]



from abc import ABC, abstractmethod
from collections.abc import Awaitable, Callable
from typing import Any, final

from baml_agents._baml_client_proxy._hooks._base_hook import (
    BaseBamlHookAsync,
    BaseBamlHookContext,
    BaseBamlHookSync,
)

PartialCallback = Callable[[Any], Awaitable[None]] | None


@final
class HandleLlmInteractionHookContext(BaseBamlHookContext):
    pass


class HandleLlmInteractionHookAsync(BaseBamlHookAsync, ABC):
    @abstractmethod
    async def handle_llm_interaction(
        self,
        context: HandleLlmInteractionHookContext,
    ) -> Any: ...


class HandleLlmInteractionHookSync(BaseBamlHookSync, ABC):
    @abstractmethod
    def handle_llm_interaction(
        self,
        context: HandleLlmInteractionHookContext,
    ) -> Any: ...



#!/usr/bin/env -S uv run

import argparse
import sys
import traceback
from pathlib import Path

import regex as re

# Regex to find a test block. Captures:
# Group 1: test header (e.g., "test MyTestName {")
# Group 2: test name (e.g., "MyTestName")
# Group 3: content inside the test block's main braces (recursive)
# Group 4: test footer (closing brace, e.g., "}")
TEST_BLOCK_PATTERN = re.compile(
    r"^(^\s*test\s+([\w\d_-]+)\s*\{)"  # Group 1 (header), Group 2 (name)
    r"((?:[^{}]|\{(?3)\})*)"  # Group 3 (content, handles balanced braces)
    r"(^\s*\})",  # Group 4 (closing brace)
    re.VERBOSE | re.MULTILINE | re.DOTALL,
)

# Regex to find an args block within a test block's content. Captures:
# Group 1: args header (e.g., "args {")
# Group 2: content inside the args block's braces (recursive)
# Group 3: args footer (closing brace, e.g., "}")
ARGS_BLOCK_PATTERN = re.compile(
    r"(\bargs\s*\{)"  # Group 1 (header)
    r"((?:[^{}]|\{(?2)\})*)"  # Group 2 (content, handles balanced braces)
    r"(\})",  # Group 3 (closing brace)
    re.VERBOSE | re.DOTALL,
)

# Regex for the specific file input pattern: arg_name { file "path/to/file.ext" }
# It captures:
# Group 1: arg_name (e.g., "input_csv")
# Group 2: Double-quoted path (e.g., "./data.csv")
# Group 3: Single-quoted path
# Group 4: Unquoted path
FILE_INPUT_PATTERN = re.compile(
    r"""
    (\b[\w\d_-]+\b)  # Group 1: arg_name
    \s*
    (?:               # Non-capturing group for the whole { file ... } structure
        \{
        \s*
        file
        \s+
        (?:           # Non-capturing group for path alternatives
            "([^"]*)"    # Group 2: Double-quoted path
            |
            '([^']*)'    # Group 3: Single-quoted path
            |
            ([^#\"'\s][^\s,}]*) # Group 4: Unquoted path (cannot start with #, ", ' or whitespace; cannot contain whitespace, ,, })
        )
        \s*
        \}
    )
    """,
    re.VERBOSE,
)


def make_baml_string_literal(content: str) -> str:
    """Creates a BAML string literal, using #"..."# syntax, handling internal #", "# etc."""
    if not content:
        return '#""#'  # BAML representation of an empty string

    num_hashes = 1
    # Increment hashes if the chosen delimiter would conflict with the content
    while f'{"#" * num_hashes}"' in content or f'"{"#" * num_hashes}' in content:
        num_hashes += 1

    hashes = "#" * num_hashes
    return f'{hashes}"{content}"{hashes}'


def _process_one_args_block_inner_content(
    args_inner_content: str, baml_file_path: Path, test_name: str, *, verbose: bool
) -> tuple[str, bool]:  # Returns (modified_content, was_modified)
    """Helper to process the inner content of a single args block."""
    current_args_block_modified = False

    def file_replacer_func(file_input_match_obj):
        nonlocal current_args_block_modified
        arg_name = file_input_match_obj.group(1)

        relative_file_path_str = (
            file_input_match_obj.group(2)  # Double-quoted
            or file_input_match_obj.group(3)  # Single-quoted
            or file_input_match_obj.group(4)  # Unquoted
        )

        if not relative_file_path_str:
            if verbose:
                print(
                    f"      Warning: Could not extract file path for arg '{arg_name}' in test '{test_name}' of {baml_file_path}. Match: {file_input_match_obj.group(0)}",
                    file=sys.stderr,
                )
            return file_input_match_obj.group(0)

        data_file_path = baml_file_path.parent / relative_file_path_str.strip()
        if verbose:
            print(
                f"      Found file reference for '{arg_name}' in test '{test_name}': '{data_file_path}' (original relative: '{relative_file_path_str.strip()}')",
                file=sys.stderr,
            )

        try:
            file_content = data_file_path.read_text(encoding="utf-8")
            baml_string = make_baml_string_literal(file_content)
            replacement = f"{arg_name} {baml_string}"
            current_args_block_modified = True
            if verbose:
                print(
                    f"        Inlined content for '{arg_name}'. Original: '{file_input_match_obj.group(0)}', New: '{replacement[:50]}...'",
                    file=sys.stderr,
                )
            return replacement
        except FileNotFoundError:
            if verbose:
                print(
                    f"      Error: Data file not found for arg '{arg_name}' in test '{test_name}': {data_file_path}",
                    file=sys.stderr,
                )
        except Exception as e:
            if verbose:
                print(
                    f"      Error reading or processing data file for arg '{arg_name}' in test '{test_name}' ({data_file_path}): {e}",
                    file=sys.stderr,
                )
                traceback.print_exc(file=sys.stderr)
        return file_input_match_obj.group(0)  # Return original on error

    modified_args_inner_content = FILE_INPUT_PATTERN.sub(
        file_replacer_func, args_inner_content
    )
    return modified_args_inner_content, current_args_block_modified


def process_baml_file_content(
    content: str,
    baml_file_path: Path,
    *,
    verbose: bool,
    test_name_prefix: str,
    test_name_suffix: str,
) -> list[str]:
    """
    Processes BAML content to inline file references in test args.
    Returns a list of strings, where each string is a fully reconstructed
    test block that had its content modified (inlined) and name changed.
    """
    collected_modified_test_blocks = []

    for test_match_obj in TEST_BLOCK_PATTERN.finditer(content):
        original_test_header = test_match_obj.group(1)
        original_test_name = test_match_obj.group(2)
        test_inner_content = test_match_obj.group(3)
        test_footer = test_match_obj.group(4)

        if verbose:
            print(f"  Processing test block: {original_test_name}", file=sys.stderr)

        current_test_block_was_modified = False  # Flag for this specific test block

        def replace_args_block_match(
            args_match_obj, original_test_name=original_test_name
        ):
            nonlocal current_test_block_was_modified
            args_header = args_match_obj.group(1)
            original_args_inner_content = args_match_obj.group(2)
            args_footer = args_match_obj.group(3)

            if verbose:
                print(
                    f"    Found args block in '{original_test_name}'.", file=sys.stderr
                )

            processed_args_inner_content, args_block_had_inlining = (
                _process_one_args_block_inner_content(
                    original_args_inner_content,
                    baml_file_path,
                    original_test_name,
                    verbose=verbose,
                )
            )

            if args_block_had_inlining:
                current_test_block_was_modified = True
                return f"{args_header}{processed_args_inner_content}{args_footer}"
            return args_match_obj.group(0)  # Return original args block if not modified

        modified_test_inner_content = ARGS_BLOCK_PATTERN.sub(
            replace_args_block_match, test_inner_content
        )

        if current_test_block_was_modified:
            new_test_name = f"{test_name_prefix}{original_test_name}{test_name_suffix}"
            # Replace the old test name in the header with the new one
            new_test_header = re.sub(
                r"\b" + re.escape(original_test_name) + r"\b",
                new_test_name,
                original_test_header,
                count=1,
            )

            reconstructed_block = (
                f"{new_test_header}{modified_test_inner_content}{test_footer}"
            )
            collected_modified_test_blocks.append(reconstructed_block)
            if verbose:
                print(
                    f"    Test block '{original_test_name}' was modified, renamed to '{new_test_name}', and collected.",
                    file=sys.stderr,
                )
        elif verbose:
            print(
                f"    Test block '{original_test_name}' had no inlinable file references.",
                file=sys.stderr,
            )

    return collected_modified_test_blocks


def inline_baml_tests():
    parser = argparse.ArgumentParser(
        description="Recursively find .baml files, inline 'file' references in test args, "
        "rename these tests, and write them to a consolidated output file."
    )
    parser.add_argument(
        "--search-root-path",
        default=str(Path.cwd()),
        help="The root folder path to search for .baml files. Defaults to the current working directory.",
    )
    parser.add_argument(
        "--output-folder",
        default=".inlined/",
        help="Folder to write the consolidated inlined tests file. Default: '.inlined/'",
    )
    parser.add_argument(
        "--test-name-prefix",
        default="",
        help="Prefix to add to the names of inlined test functions. Default: empty string.",
    )
    parser.add_argument(
        "--test-name-suffix",
        default="_inlined",
        help="Suffix to add to the names of inlined test functions. Default: '_inlined'.",
    )
    parser.add_argument(
        "--verbose",
        choices=["true", "false"],
        default="false",
        help="Set to 'true' for detailed output to stderr, 'false' for summary only. Default: 'false'.",
    )

    args = parser.parse_args()

    root_folder = Path(args.search_root_path)
    verbose = args.verbose.lower() == "true"
    test_name_prefix = args.test_name_prefix
    test_name_suffix = args.test_name_suffix
    output_folder_path = Path(args.output_folder)

    output_filename = "inlined_tests.baml"  # Fixed name for the output file
    output_file_path = output_folder_path / output_filename

    if not root_folder.is_dir():
        print(
            f"Error: Root search folder not found or is not a directory: {root_folder}",
            file=sys.stderr,
        )
        sys.exit(1)

    if verbose:
        print(f"Starting search in: {root_folder}", file=sys.stderr)
        print(f"Output target: {output_file_path}", file=sys.stderr)
        print(f"Test name prefix: '{test_name_prefix}'", file=sys.stderr)
        print(f"Test name suffix: '{test_name_suffix}'", file=sys.stderr)
        print("-" * 30, file=sys.stderr)

    processed_files_count = 0
    total_inlined_tests_collected = 0
    files_with_inlined_tests = 0

    all_collected_inlined_test_blocks = (
        []
    )  # To store all modified test blocks as strings

    for baml_file_path in root_folder.rglob("*.baml"):
        if not baml_file_path.is_file():
            continue

        processed_files_count += 1
        if verbose:
            print(f"Processing: {baml_file_path}", file=sys.stderr)

        try:
            original_content = baml_file_path.read_text(encoding="utf-8")
            modified_test_blocks_from_current_file = process_baml_file_content(
                original_content,
                baml_file_path,
                verbose=verbose,
                test_name_prefix=test_name_prefix,
                test_name_suffix=test_name_suffix,
            )

            if modified_test_blocks_from_current_file:
                files_with_inlined_tests += 1
                all_collected_inlined_test_blocks.extend(
                    modified_test_blocks_from_current_file
                )
                num_collected_this_file = len(modified_test_blocks_from_current_file)
                total_inlined_tests_collected += num_collected_this_file
                if verbose:
                    print(
                        f"  Collected {num_collected_this_file} inlined test(s) from {baml_file_path}.",
                        file=sys.stderr,
                    )
            elif verbose:
                print(
                    f"  No inlinable test arguments found in {baml_file_path}.",
                    file=sys.stderr,
                )

        except Exception as e:
            print(
                f"  An unexpected error occurred processing {baml_file_path}: {e}",
                file=sys.stderr,
            )
            if verbose:
                traceback.print_exc(file=sys.stderr)

        if verbose and modified_test_blocks_from_current_file:
            print(file=sys.stderr)  # Visual separator in verbose log for multiple files

    output_file_written_successfully = False
    if all_collected_inlined_test_blocks:
        folder_was_brand_new = not output_folder_path.exists()
        try:
            # Ensure the output directory exists
            # mkdir(exist_ok=True) will not error if it's already a directory.
            # It WILL error with FileExistsError if output_folder_path is a file.
            output_folder_path.mkdir(parents=True, exist_ok=True)
            if folder_was_brand_new and verbose:
                print(f"Created output folder: {output_folder_path}", file=sys.stderr)

            # If the folder was newly created, add .gitignore
            if folder_was_brand_new:
                gitignore_path = output_folder_path / ".gitignore"
                try:
                    gitignore_path.write_text(
                        "*\n# Content of this folder is auto-generated.\n",
                        encoding="utf-8",
                    )
                    if verbose:
                        print(
                            f"Created .gitignore in {output_folder_path}",
                            file=sys.stderr,
                        )
                except Exception as e_git:
                    print(
                        f"Warning: Could not create .gitignore in {output_folder_path}: {e_git}",
                        file=sys.stderr,
                    )

            # Write the consolidated BAML content
            try:
                with output_file_path.open("w", encoding="utf-8") as f_out:
                    f_out.write("\n\n".join(all_collected_inlined_test_blocks))
                    f_out.write("\n")  # Ensure final newline
                if verbose:
                    print(
                        f"Successfully wrote {total_inlined_tests_collected} inlined test(s) to: {output_file_path}",
                        file=sys.stderr,
                    )
                output_file_written_successfully = True
            except Exception as e_write:
                print(
                    f"Error: Could not write to output file {output_file_path}: {e_write}",
                    file=sys.stderr,
                )
                if verbose:
                    traceback.print_exc(file=sys.stderr)

        except FileExistsError:  # Raised by mkdir if output_folder_path is a file
            print(
                f"Error: Output path '{output_folder_path}' exists but is a file, not a directory. Cannot write output.",
                file=sys.stderr,
            )
        except OSError as e_dir:  # Other OS errors during directory creation
            print(
                f"Error: Could not create or access output folder '{output_folder_path}': {e_dir}",
                file=sys.stderr,
            )

    # Final summary messages
    if verbose:
        print("-" * 30, file=sys.stderr)
    print(
        f"Finished. Processed {processed_files_count} .baml file(s).", file=sys.stderr
    )
    print(
        f"Collected {total_inlined_tests_collected} inlined test function(s) from {files_with_inlined_tests} file(s).",
        file=sys.stderr,
    )

    if total_inlined_tests_collected > 0:
        if output_file_written_successfully:
            print(
                f"Consolidated output written to: {output_file_path}", file=sys.stderr
            )
        else:
            print(
                f"Consolidated output was NOT written due to errors (see messages above). Target path: {output_file_path}",
                file=sys.stderr,
            )
    else:
        print(
            f"No inlined tests to write. Output file '{output_file_path}' was not created.",
            file=sys.stderr,
        )


if __name__ == "__main__":
    inline_baml_tests()



from typing import TYPE_CHECKING, TypeVar

from baml_agents._baml_client_proxy._baml_client_proxy import BamlClientProxy

if TYPE_CHECKING:
    from collections.abc import Sequence

    from baml_agents._baml_client_proxy._hooks._base_hook import BaseBamlHook

T_BamlClient = TypeVar("T_BamlClient")


def with_hooks(
    b: T_BamlClient,
    hooks: "Sequence[BaseBamlHook]",
) -> T_BamlClient:
    """
    Applies lifecycle hooks to a BAML client instance by wrapping it.

    Args:
        b: The original baml_client instance.
        hooks: A list of BamlHook instances.

    Returns:
        A BamlClient wrapper instance that provides the same interface
        as the original client but executes hooks.

    """
    if isinstance(b, BamlClientProxy):
        return b.add_hooks(hooks)
    return BamlClientProxy(b, hooks=hooks)  # type: ignore
